PRAGMA foreign_keys = ON;
BEGIN TRANSACTION;

--涉密功能：创建记录表
CREATE TABLE objects_list_sm(
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	path TEXT NOT NULL UNIQUE,
	hash TEXT NOT NULL,
	type TEXT NOT NULL CHECK(type in ('program','library','kmodule','script','other')),
	exectl TEXT NOT NULL CHECK(exectl in ('original', 'verified', 'kysoft', 'trusted', 'parent', 'script', 'unknown')),
	filepro TEXT NOT NULL CHECK(filepro in ('none', 'readonly')),
	rmmodpro INTEGER NOT NULL DEFAULT 0 CHECK(rmmodpro in (0,1)),
	status INTEGER NOT NULL DEFAULT 0, --执行控制或内核模块保护状态：0 正常，1 已篡改，2不存在，3 已解除
	sm_status INTEGER NOT NULL DEFAULT 0 --涉密状态： 0 非涉密数据 1 涉密数据
);
--通用功能
CREATE TABLE relabel_status(
	relabel INTEGER NOT NULL DEFAULT 0 --标记初始化标识： 0 全盘初始化，1 按照白名单初始化，2 重新初始化执行控制白名单
);

--涉密功能
CREATE TABLE IF NOT EXISTS kmod_list_sm(
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    kmod_name TEXT NOT NULL UNIQUE,
    status INTEGER NOT NULL DEFAULT 0 CHECK(status in (0,1)),
    sm_status INTEGER NOT NULL DEFAULT 0, --涉密状态： 0 非涉密数据 1 涉密数据    
    ctime INTEGER  NOT NULL DEFAULT (strftime('%s','now'))
);

--通用功能
CREATE VIEW IF NOT EXISTS kmod_whlist AS 
SELECT id, kmod_name, status, ctime from kmod_list_sm where sm_status = 0;

--通用功能
CREATE TRIGGER IF NOT EXISTS kmod_whlist_delete 
INSTEAD OF DELETE ON kmod_whlist 
BEGIN 
	UPDATE kmod_list_sm SET status = 0, ctime = strftime('%s','now') WHERE kmod_name=OLD.kmod_name and sm_status = 0;
END;

--通用功能
CREATE TRIGGER IF NOT EXISTS kmod_whlist_insert_exists 
INSTEAD OF INSERT ON kmod_whlist WHEN EXISTS(select 1 from kmod_list_sm where kmod_name=NEW.kmod_name) 
BEGIN 
	UPDATE kmod_list_sm SET status = NEW.status, ctime = strftime('%s','now') WHERE kmod_name=NEW.kmod_name and sm_status = 0;
END;

--通用功能
CREATE TRIGGER IF NOT EXISTS kmod_whlist_insert_not_exists 
INSTEAD OF INSERT ON kmod_whlist WHEN NOT EXISTS(select 1 from kmod_list_sm where kmod_name=NEW.kmod_name) 
BEGIN 
    INSERT INTO kmod_list_sm(kmod_name, status) VALUES(NEW.kmod_name, NEW.status);
END;

--涉密功能
CREATE VIEW IF NOT EXISTS kmod_whlist_sm AS 
SELECT id, kmod_name, status, ctime from kmod_list_sm;

--涉密功能
CREATE TRIGGER IF NOT EXISTS kmod_whlist_sm_delete 
INSTEAD OF DELETE ON kmod_whlist_sm 
BEGIN
	UPDATE kmod_list_sm SET status = 0, sm_status = 0, ctime = strftime('%s','now') WHERE kmod_name=OLD.kmod_name;
END;

--涉密功能
CREATE TRIGGER IF NOT EXISTS kmod_whlist_sm_insert_exists
INSTEAD OF INSERT ON kmod_whlist_sm WHEN EXISTS(select 1 from kmod_list_sm where kmod_name=NEW.kmod_name) 
BEGIN
	UPDATE kmod_list_sm SET status = NEW.status, sm_status = NEW.status, ctime = strftime('%s','now') WHERE kmod_name=NEW.kmod_name; 
END;

--涉密功能
CREATE TRIGGER IF NOT EXISTS kmod_whlist_sm_insert_not_exists 
INSTEAD OF INSERT ON kmod_whlist_sm WHEN NOT EXISTS(select 1 from kmod_list_sm where kmod_name=NEW.kmod_name) 
BEGIN 
    INSERT INTO kmod_list_sm(kmod_name, status, sm_status) VALUES(NEW.kmod_name, NEW.status, 1);
END;

DELETE from relabel_status;
INSERT into relabel_status(relabel) values(0);

--涉密功能：执行控制功能白名单视图
CREATE VIEW exectl_whlist_sm AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list_sm where type in ('program', 'library', 'script') and status != 2;

--涉密功能：文件保护功能视图
CREATE VIEW filepro_whlist_sm AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list_sm where filepro = 'readonly';

--涉密功能：内核模块保护功能白名单视图
CREATE VIEW kmodpro_whlist_sm AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list_sm where type in ('kmodule') and status != 2;

--通用功能：创建记录视图
CREATE VIEW objects_list AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list_sm where sm_status = 0;

--通用功能：执行控制功能白名单视图
 CREATE VIEW exectl_whlist AS
 SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list where type in ('program', 'library', 'script') and status != 2;
 
--文件保护功能视图
--通用功能：文件保护功能视图
 CREATE VIEW filepro_whlist AS
 SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list where filepro = 'readonly';
 
--内核模块保护功能白名单视图
--通用功能：内核模块保护功能白名单视图
CREATE VIEW kmodpro_whlist AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list where type in ('kmodule') and status != 2;
 
COMMIT;

--记录更新触发器，更新记录status字段时，若status状态为0，则删除该记录
--通用功能：记录表插入不存在的记录
CREATE TRIGGER objects_list_insert_not_exists
INSTEAD OF INSERT ON objects_list WHEN NOT EXISTS(select 1 from objects_list_sm where path=NEW.path)
BEGIN
	INSERT INTO objects_list_sm(path, hash, type, exectl, filepro, rmmodpro, status, sm_status) values(NEW.path, NEW.hash, NEW.type, NEW.exectl, NEW.filepro, NEW.rmmodpro, NEW.status, 0);
END;

--通用功能：记录表插入存在的记录
CREATE TRIGGER objects_list_insert_exists
INSTEAD OF INSERT ON objects_list WHEN EXISTS(select 1 from objects_list_sm where path=NEW.path)
BEGIN
	UPDATE objects_list_sm SET hash = NEW.hash, type = NEW.type, exectl = NEW.exectl, filepro = NEW.filepro, rmmodpro = NEW.rmmodpro, status = NEW.status WHERE path=NEW.path and sm_status = 0;
END;

--通用功能：记录表删除记录
CREATE TRIGGER objects_list_delete
INSTEAD OF DELETE ON objects_list
BEGIN
	DELETE FROM objects_list_sm where path = OLD.path and sm_status = 0;
END;

--通用功能：记录表更新记录
CREATE TRIGGER objects_list_update
INSTEAD OF UPDATE ON objects_list
BEGIN
	UPDATE objects_list_sm SET hash = NEW.hash, type = NEW.type, exectl = NEW.exectl, filepro = NEW.filepro, rmmodpro = NEW.rmmodpro, status = NEW.status WHERE path=NEW.path and sm_status = 0;
END;

--涉密功能：记录更新触发器，更新记录status字段时，若status状态为0，则删除该记录
CREATE TRIGGER objects_list_sm_delete AFTER UPDATE OF filepro
ON objects_list_sm
BEGIN
	DELETE FROM objects_list_sm where id = new.id and filepro = 'none' and type = 'other';
END;

--涉密功能：执行控制功能白名单视图删除触发器，采用修改exectl字段值为'unknown'替换, 并修改status值为0
CREATE TRIGGER exectl_whlist_sm_delete
INSTEAD OF DELETE ON exectl_whlist_sm
BEGIN
	UPDATE objects_list_sm SET exectl = 'unknown', status = 3, sm_status = 0 WHERE path=OLD.path;
END;

--涉密功能：执行控制功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态， 并修改status值为1
CREATE TRIGGER exectl_whlist_sm_insert_exists
INSTEAD OF INSERT ON exectl_whlist_sm WHEN EXISTS(select 1 from objects_list_sm where path=NEW.path)
BEGIN
	UPDATE objects_list_sm SET exectl = NEW.exectl, hash = NEW.hash, status = 0, sm_status = 1 WHERE path=NEW.path;
END;

--涉密功能：执行控制功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER exectl_whlist_sm_insert_not_exists
INSTEAD OF INSERT ON exectl_whlist_sm WHEN NOT EXISTS(select 1 from objects_list_sm where path=NEW.path)
BEGIN
	INSERT INTO objects_list_sm(path, hash, type, exectl, filepro, rmmodpro, status, sm_status) values(NEW.path, NEW.hash, NEW.type, NEW.exectl, 'none', 0, 0, 1);
END;

--涉密功能：文件保护功能视图删除触发器，采用修改filepro字段值为none替换， 并修改status值为0
CREATE TRIGGER filepro_whlist_sm_delete
INSTEAD OF DELETE ON filepro_whlist_sm
BEGIN
	UPDATE objects_list_sm SET filepro = 'none', sm_status = 0 WHERE path=OLD.path;	
END;

--涉密功能：文件保护功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态， 并修改status值为1
CREATE TRIGGER filepro_whlist_sm_insert_exists
INSTEAD OF INSERT ON filepro_whlist_sm WHEN EXISTS(select 1 from objects_list_sm where path=NEW.path)
BEGIN
	UPDATE objects_list_sm SET filepro = NEW.filepro, sm_status = 1 WHERE path=NEW.path;
END;

--涉密功能：文件保护功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER filepro_whlist_sm_insert_not_exists
INSTEAD OF INSERT ON filepro_whlist_sm WHEN NOT EXISTS(select 1 from objects_list_sm where path=NEW.path)
BEGIN
	INSERT INTO objects_list_sm(path, hash, type, exectl, filepro, rmmodpro, status, sm_status) values(NEW.path, NEW.hash, NEW.type, 'unknown', NEW.filepro, 0, 0, 1);
END;

--涉密功能：内核模块保护功能白名单视图删除触发器，采用修改exectl字段为unknown，rmmodpro字段值为0替换，并修改status值为0
CREATE TRIGGER kmodpro_whlist_sm_delete
INSTEAD OF DELETE ON kmodpro_whlist_sm
BEGIN
	UPDATE objects_list_sm SET exectl = 'unknown', status = 0, sm_status = 0 WHERE path=OLD.path;
END;

--涉密功能：内核模块保护功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态， 并修改status值为1
CREATE TRIGGER kmodpro_whlist_sm_insert_exists
INSTEAD OF INSERT ON kmodpro_whlist_sm WHEN EXISTS(select 1 from objects_list_sm where path=NEW.path)
BEGIN
	UPDATE objects_list_sm SET exectl = NEW.exectl, hash = NEW.hash, status = 0, sm_status = 1 WHERE path=NEW.path;
END;

--涉密功能：内核模块保护功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER kmodpro_whlist_sm_insert_not_exists
INSTEAD OF INSERT ON kmodpro_whlist_sm WHEN NOT EXISTS(select 1 from objects_list_sm where path=NEW.path)
BEGIN
	INSERT INTO objects_list_sm(path, hash, type, exectl, filepro, rmmodpro, status, sm_status) values(NEW.path, NEW.hash, NEW.type, NEW.exectl, 'none', 0, 0, 1);
END;

--通用功能：执行控制功能白名单视图删除触发器，采用修改exectl字段值为'unknown'替换, 并修改status值为0
CREATE TRIGGER exectl_whlist_delete
INSTEAD OF DELETE ON exectl_whlist
BEGIN
	UPDATE objects_list SET exectl = 'unknown', status = 0 WHERE path=OLD.path;
END;

--通用功能：执行控制功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态， 并修改status值为1
CREATE TRIGGER exectl_whlist_insert_exists
INSTEAD OF INSERT ON exectl_whlist WHEN EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	UPDATE objects_list SET exectl = NEW.exectl, hash = NEW.hash, status = 0 WHERE path=NEW.path;
END;

--通用功能：执行控制功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER exectl_whlist_insert_not_exists
INSTEAD OF INSERT ON exectl_whlist WHEN NOT EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	INSERT INTO objects_list(path, hash, type, exectl, filepro, rmmodpro, status) values(NEW.path, NEW.hash, NEW.type, NEW.exectl, 'none', 0, 0);
END;

--通用功能：文件保护功能视图删除触发器，采用修改filepro字段值为none替换， 并修改status值为0
CREATE TRIGGER filepro_whlist_delete
INSTEAD OF DELETE ON filepro_whlist
BEGIN
	UPDATE objects_list SET filepro = 'none' WHERE path=OLD.path;	
END;

--通用功能：文件保护功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态， 并修改status值为1
CREATE TRIGGER filepro_whlist_insert_exists
INSTEAD OF INSERT ON filepro_whlist WHEN EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	UPDATE objects_list SET filepro = NEW.filepro WHERE path=NEW.path;
END;

--通用功能：文件保护功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER filepro_whlist_insert_not_exists
INSTEAD OF INSERT ON filepro_whlist WHEN NOT EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	INSERT INTO objects_list(path, hash, type, exectl, filepro, rmmodpro, status) values(NEW.path, NEW.hash, NEW.type, 'unknown', NEW.filepro, 0, 0);
END;

--通用功能：内核模块保护功能白名单视图删除触发器，采用修改exectl字段为unknown，rmmodpro字段值为0替换，并修改status值为0
CREATE TRIGGER kmodpro_whlist_delete
INSTEAD OF DELETE ON kmodpro_whlist
BEGIN
	UPDATE objects_list SET exectl = 'unknown', status = 0 WHERE path=OLD.path;
END;

--通用功能：内核模块保护功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态， 并修改status值为1
CREATE TRIGGER kmodpro_whlist_insert_exists
INSTEAD OF INSERT ON kmodpro_whlist WHEN EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	UPDATE objects_list SET exectl = NEW.exectl, hash = NEW.hash, status = 0 WHERE path=NEW.path;
END;

--通用功能：内核模块保护功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER kmodpro_whlist_insert_not_exists
INSTEAD OF INSERT ON kmodpro_whlist WHEN NOT EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	INSERT INTO objects_list(path, hash, type, exectl, filepro, rmmodpro, status) values(NEW.path, NEW.hash, NEW.type, NEW.exectl, 'none', 0, 0);
END;
